#!/usr/bin/perl
# 
# Plug this into openvpn.conf as the client-connect script
# and dynamically add/remove iptables rules
#
# @author Joel Heenan 29/04/2010

use strict;
use warnings;
use Log::Log4perl qw(get_logger :levels);
use vars qw($logger);
use Readonly;
use IPTables::IPv4;

# *** evil constants ***

#logging
Readonly::Scalar our $script_log_file => '/var/log/openvpn_iptables.log';
Readonly::Scalar our $DEBUG           => 0;


sub initialize_logger($)
{
  my $debug = shift;

  my $level = 'INFO';

  if ($debug)
  {
    $level = 'DEBUG';
  }

  my $conf = qq(
    log4perl.rootLogger                = $level, Logfile, Screen

    log4perl.appender.Logfile          = Log::Log4perl::Appender::File
    log4perl.appender.Logfile.filename = $script_log_file
    log4perl.appender.Logfile.layout   = Log::Log4perl::Layout::PatternLayout
    log4perl.appender.Logfile.layout.ConversionPattern = %d %p> %F{1}:%L %M - %m%n
    log4perl.appender.Logfile.level    = DEBUG
    log4perl.appender.Logfile.mode     = append

    log4perl.appender.Screen         = Log::Log4perl::Appender::Screen
    log4perl.appender.Screen.layout  = Log::Log4perl::Layout::SimpleLayout
    log4perl.appender.Screen.level   = $level
  );


  Log::Log4perl::init( \$conf );

  $logger = get_logger();

  return;
}

sub remove_iptables_rules($)
{
  my $remote_ip = shift;

  my $natt = IPTables::IPv4::init('nat');

  $logger->debug("Cleaning up after ourselves");

  $natt->delete_entry("POSTROUTING", {
    "source"  => $remote_ip,
    "jump"    => $remote_ip,
  });

  if( $natt->is_chain($remote_ip) )
  {
    $logger->debug("Deleting chain $remote_ip");
    $natt->flush_entries($remote_ip);
    $natt->delete_chain($remote_ip)
      or $logger->logdie("Couldn't delete chain $remote_ip: $!");
  }

  $natt->commit();
}

sub main()
{
  initialize_logger($DEBUG);
  my $remote_ip = $ENV{'ifconfig_pool_remote_ip'};
  $logger->info(sprintf("Deleting iptables rules for IP %s...",$remote_ip));
  remove_iptables_rules($remote_ip);
}

main();

__END__
